"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var frint_di_1 = require("frint-di");
var find = require("lodash/find");
var findIndex = require("lodash/findIndex");
var get = require("lodash/get");
var omit = require("lodash/omit");
var BehaviorSubject_1 = require("rxjs/BehaviorSubject");
var of_1 = require("rxjs/observable/of");
var concatMap_1 = require("rxjs/operators/concatMap");
var find_1 = require("rxjs/operators/find");
var first_1 = require("rxjs/operators/first");
var map_1 = require("rxjs/operators/map");
function makeInstanceKey(region, regionKey, multi) {
    if (region === void 0) { region = null; }
    if (regionKey === void 0) { regionKey = null; }
    if (multi === void 0) { multi = false; }
    if (!multi ||
        (!region && !regionKey)) {
        return 'default';
    }
    var key = '';
    if (region) {
        key = region;
    }
    if (regionKey) {
        key = region + "_" + regionKey;
    }
    return key;
}
var defaultProviderNames = {
    component: 'component',
    container: 'container',
    store: 'store',
    app: 'app',
    parentApp: 'parentApp',
    rootApp: 'rootApp',
    region: 'region',
};
var App = /** @class */ (function () {
    function App(opts) {
        var _this = this;
        this.options = __assign({ name: null, methods: {}, parentApp: null, providers: [], providerNames: defaultProviderNames, 
            // lifecycle callbacks
            // tslint:disable-next-line:no-empty
            initialize: function () { }, 
            // tslint:disable-next-line:no-empty
            beforeDestroy: function () { } }, opts);
        // errors
        if (!this.options.name) {
            throw new Error('Must provide `name` in options');
        }
        // expose methods as class properties
        Object.keys(this.options.methods).forEach(function (methodName) {
            var method = _this.options.methods[methodName];
            if (typeof method === 'function') {
                if (_this[methodName] !== undefined) {
                    throw new Error("Cannot overwrite app's `" + methodName + "` property or method with options method.");
                }
                _this[methodName] = method.bind(_this);
            }
        });
        // children - create Observable if root
        this._appsCollection = [];
        this._apps$ = new BehaviorSubject_1.BehaviorSubject(this._appsCollection);
        // container
        var container = frint_di_1.createContainer([
            { name: this.options.providerNames.app, useDefinedValue: this },
            { name: this.options.providerNames.parentApp, useDefinedValue: this.getParentApp() },
            { name: this.options.providerNames.rootApp, useDefinedValue: this.getRootApp() },
        ], {
            containerName: this.options.providerNames.container,
        });
        this.container = frint_di_1.resolveContainer(container);
        // root app's providers
        this._registerRootProviders();
        // self providers
        this.options.providers.forEach(function (provider) {
            _this.container.register(provider);
        });
        this.options.initialize.bind(this)();
    }
    App.prototype.getContainer = function () {
        return this.container;
    };
    App.prototype.getRootApp = function () {
        var parents = this.getParentApps();
        if (parents.length === 0) {
            return this;
        }
        return parents.pop();
    };
    App.prototype.getParentApp = function () {
        return this.options[this.options.providerNames.parentApp] || null;
    };
    App.prototype.getParentApps = function () {
        function findParents(app, parents) {
            if (parents === void 0) { parents = []; }
            var parentApp = app.getParentApp();
            if (!parentApp) {
                return parents;
            }
            parents.push(parentApp);
            return findParents(parentApp, parents);
        }
        return findParents(this);
    };
    App.prototype.getOption = function (key) {
        return get(this.options, key);
    };
    App.prototype.getName = function () {
        return this.getOption('name');
    };
    App.prototype.getProviders = function () {
        return this.options.providers;
    };
    App.prototype.getProvider = function (name) {
        return find(this.options.providers, function (p) {
            return p.name === name;
        });
    };
    App.prototype.get = function (providerName) {
        var value = this.container.get(providerName);
        if (typeof value !== 'undefined') {
            return value;
        }
        return null;
    };
    App.prototype.getApps$ = function (regionName) {
        if (regionName === void 0) { regionName = null; }
        if (!regionName) {
            return this._apps$;
        }
        return this._apps$
            .pipe(map_1.map(function (collection) {
            return collection
                .filter(function (w) {
                return w.regions.indexOf(regionName) > -1;
            });
        }));
    };
    App.prototype.registerApp = function (appClass, opts) {
        if (appClass === void 0) { appClass = App; }
        if (opts === void 0) { opts = {}; }
        var options = __assign({ multi: false }, opts);
        if (typeof options.name !== 'undefined') {
            Object.defineProperty(appClass, 'frintAppName', {
                value: options.name,
                configurable: true,
            });
        }
        var existingIndex = findIndex(this._appsCollection, function (w) {
            return w.name === appClass.frintAppName;
        });
        if (existingIndex !== -1) {
            throw new Error("App '" + appClass.frintAppName + "' has been already registered before.");
        }
        this._appsCollection.push(__assign({}, options, { name: appClass.frintAppName, appClass: appClass, instances: {}, regions: options.regions || [] }));
        if (options.multi === false) {
            this.instantiateApp(appClass.frintAppName);
        }
        this._apps$.next(this._appsCollection);
    };
    App.prototype.hasAppInstance = function (name, region, regionKey) {
        if (region === void 0) { region = null; }
        if (regionKey === void 0) { regionKey = null; }
        var instance = this.getAppInstance(name, region, regionKey);
        if (instance && typeof instance !== 'undefined') {
            return true;
        }
        return false;
    };
    App.prototype.getAppInstance = function (name, region, regionKey) {
        if (region === void 0) { region = null; }
        if (regionKey === void 0) { regionKey = null; }
        var index = findIndex(this._appsCollection, function (a) {
            return a.name === name;
        });
        if (index === -1) {
            return null;
        }
        var app = this._appsCollection[index];
        var instanceKey = makeInstanceKey(region, regionKey, app.multi);
        var instance = app.instances[instanceKey];
        if (!instance || typeof instance === 'undefined') {
            return null;
        }
        return instance;
    };
    App.prototype.getAppOnceAvailable$ = function (name, region, regionKey) {
        if (region === void 0) { region = null; }
        if (regionKey === void 0) { regionKey = null; }
        var rootApp = this.getRootApp();
        var w = rootApp.getAppInstance(name, region, regionKey);
        if (w) {
            return of_1.of(w);
        }
        return rootApp._apps$
            .pipe(concatMap_1.concatMap(function (y) { return y; }), find_1.find(function (app) { return app.name === name; }), map_1.map(function (x) {
            var instanceKey = makeInstanceKey(region, regionKey, x.multi);
            return x.instances[instanceKey];
        }), first_1.first());
    };
    App.prototype.instantiateApp = function (name, region, regionKey) {
        if (region === void 0) { region = null; }
        if (regionKey === void 0) { regionKey = null; }
        var index = findIndex(this._appsCollection, function (a) {
            return a.appClass.frintAppName === name;
        });
        if (index === -1) {
            throw new Error("No app found with name '" + name + "'.");
        }
        var w = this._appsCollection[index];
        var key = makeInstanceKey(region, regionKey, w.multi);
        this._appsCollection[index].instances[key] = new w.appClass(__assign({}, omit(w, ['AppClass', 'instances']), { name: w.appClass.frintAppName, parentApp: this }));
        return this._appsCollection[index].instances[key];
    };
    App.prototype.destroyApp = function (name, region, regionKey) {
        if (region === void 0) { region = null; }
        if (regionKey === void 0) { regionKey = null; }
        var index = findIndex(this._appsCollection, function (a) {
            if (!a || !a.appClass) {
                return false;
            }
            return a.appClass.frintAppName === name;
        });
        if (index === -1) {
            throw new Error("No app found with name '" + name + "'.");
        }
        var w = this._appsCollection[index];
        var key = makeInstanceKey(region, regionKey, w.multi);
        if (typeof this._appsCollection[index].instances[key] === 'undefined') {
            throw new Error("No instance with key '" + key + "' found for app with name '" + name + "'.");
        }
        this._appsCollection[index].instances[key].beforeDestroy();
        delete this._appsCollection[index].instances[key];
    };
    App.prototype.beforeDestroy = function () {
        return this.options.beforeDestroy.bind(this)();
    };
    App.prototype._registerRootProviders = function () {
        var _this = this;
        var parentApps = this.getParentApps();
        if (parentApps.length === 0) {
            return;
        }
        parentApps.reverse().forEach(function (parentApp) {
            parentApp.getProviders().forEach(function (parentProvider) {
                // do not cascade
                if (!parentProvider.cascade) {
                    return;
                }
                var definedProvider = Object.assign({}, omit(parentProvider, [
                    'useClass',
                    'useValue',
                    'useFactory'
                ]));
                // non-scoped
                if (!parentProvider.scoped) {
                    _this.container.register(__assign({}, definedProvider, { useValue: parentApp.get(parentProvider.name) }));
                    return;
                }
                // scoped
                if ('useValue' in parentProvider) {
                    // `useValue` providers have no impact with scoping
                    _this.container.register(__assign({}, definedProvider, { useValue: parentApp.get(parentProvider.name) }));
                    return;
                }
                if ('useClass' in parentProvider) {
                    _this.container.register(__assign({}, definedProvider, { useClass: parentProvider.useClass }));
                    return;
                }
                if ('useFactory' in parentProvider) {
                    _this.container.register(__assign({}, definedProvider, { useFactory: parentProvider.useFactory }));
                }
            });
        });
    };
    return App;
}());
exports.App = App;
//# sourceMappingURL=App.js.map